Name:           openmpi
Version:        1.2.7
Release:        6%{?dist}
Summary:        Open Message Passing Interface

Group:          Development/Libraries
License:        BSD
URL:            http://www.open-mpi.org/
Source0:       	http://www.open-mpi.org/software/ompi/v1.2/downloads/%{name}-%{version}.tar.bz2
Source1:	openmpi.pc.in
Source2:	openmpi.module.in
Source3:	openmpi.sh.in
Source4:	openmpi.csh.in
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
BuildRequires:  gcc-gfortran, libtool, numactl-devel
BuildRequires:  libibverbs-devel, opensm-devel, compat-dapl-devel
Requires:	%{name}-libs = %{version}-%{release}
Requires(post): /usr/sbin/alternatives, mpi-selector, rpm
Provides:       mpi
ExclusiveArch: i386 x86_64 ia64 ppc ppc64

%package libs
Summary:	Libraries used by openmpi programs
Group:		Development/Libraries
Requires:       mpi-selector, pkgconfig
Requires(post): /usr/sbin/alternatives, mpi-selector, rpm, coreutils
Requires(preun): mpi-selector, rpm

%package devel
Summary:        Development files for openmpi
Group:          Development/Libraries
Requires:       %{name} = %{version}-%{release}
Requires(post): /usr/sbin/alternatives, mpi-selector, rpm
Provides:	mpi-devel

%description
Open MPI is an open source, freely available implementation of both the 
MPI-1 and MPI-2 standards, combining technologies and resources from
several other projects (FT-MPI, LA-MPI, LAM/MPI, and PACX-MPI) in
order to build the best MPI library available.  A completely new MPI-2
compliant implementation, Open MPI offers advantages for system and
software vendors, application developers, and computer science
researchers. For more information, see http://www.open-mpi.org/ .

%description libs
Contains shared libraries used by openmpi applications

%description devel
Contains development headers and libraries for openmpi

# We only compile with gcc, but other people may want other compilers.
# Set the compiler here.
%define opt_cc gcc
# Optional CFLAGS to use with the specific compiler...gcc doesn't need any,
# so uncomment and define to use
#define opt_cc_cflags

# When dealing with multilib installations, aka the ability to run either
# i386 or x86_64 binaries on x86_64 machines, we install the native i386
# openmpi libs/compilers and the native x86_64 libs/compilers.  Obviously,
# on i386 you can only run i386, so you don't really need the -m32 flag
# to gcc in order to force 32 bit mode.  However, since we use the native
# i386 package to support i386 operation on x86_64, and since on x86_64
# the default is x86_64, the i386 package needs to force i386 mode.  This
# is true of all the multilib arches, hence the non-default arch (aka i386
# on x86_64) must force the non-default mode (aka 32 bit compile) in it's
# native-arch package (aka, when built on i386) so that it will work
# properly on the non-native arch as a multilib package (aka i386 installed
# on x86_64).  Just to be safe, we also force the default mode (aka 64 bit)
# in default arch packages (aka, the x86_64 package).  There are, however,
# some arches that don't support forcing *any* mode, those we just leave
# undefined.
%ifarch i386 ppc
%define modeflag -m32
%define mode 32
%endif
%ifarch ia64
%define mode 64
%endif
%ifarch x86_64 ppc64
%define modeflag -m64
%define mode 64
%endif

# We set this to for convenience, since this is the unique dir we use for this
# particular package, version, compiler
%define mpidir %{name}/%{version}-%{opt_cc}

%prep
%setup -q
%ifarch x86_64
XFLAGS="-fPIC"
%endif

./configure --prefix=%{_libdir}/%{mpidir} --with-libnuma=%{_libdir} \
	--with-openib=/usr --with-udapl --enable-mpirun-prefix-by-default \
	--mandir=%{_libdir}/%{mpidir}/man \
	CC=%{opt_cc} \
	LDFLAGS='-Wl,-z,noexecstack' \
	CFLAGS="%{?opt_cc_cflags} $RPM_OPT_FLAGS $XFLAGS" \
	CXXFLAGS="$RPM_OPT_FLAGS $XFLAGS" \
	FFLAGS="$RPM_OPT_FLAGS $XFLAGS"

%build
make %{?_smp_mflags}

%install
rm -rf %{buildroot}
make install DESTDIR=%{buildroot}
find %{buildroot}%{_libdir}/%{mpidir}/lib -name \*.la | xargs rm
find %{buildroot}%{_libdir}/%{mpidir}/man -type f | xargs gzip -9
ln -s mpicc.1.gz %{buildroot}%{_libdir}/%{mpidir}/man/man1/mpiCC.1.gz
rm -f %{buildroot}%{_libdir}/%{mpidir}/man/man1/mpiCC.1

# Make the pkgconfig files
mkdir -p %{buildroot}%{_libdir}/pkgconfig
sed 's#@NAME@#'%{name}'#g;s#@VERSION@#'%{version}'#g;s#@LIBDIR@#'%{_libdir}'#g;s#@CC@#'%{opt_cc}'#g;s#@MPIDIR@#'%{mpidir}'#g;s#@MODEFLAG@#'%{?modeflag}'#g' < %SOURCE1 > %{buildroot}/%{_libdir}/pkgconfig/%{name}-%{version}-%{opt_cc}.pc
sed 's#@LIBDIR@#'%{_libdir}'#g;s#@MPIDIR@#'%{mpidir}'#g;s#@MODEFLAG@#'%{?modeflag}'#g' < %SOURCE2 > %{buildroot}/%{_libdir}/%{mpidir}/lib/%{name}.module

# And do the same for the mpi-selector shell scripts
sed 's#@LIBDIR@#'%{_libdir}/%{mpidir}/lib'#g;s#@BINDIR@#'%{_libdir}/%{mpidir}/bin'#g' < %SOURCE3 > %{buildroot}%{_libdir}/%{mpidir}/etc/mpivars.sh
sed 's#@LIBDIR@#'%{_libdir}/%{mpidir}/lib'#g;s#@BINDIR@#'%{_libdir}/%{mpidir}/bin'#g' < %SOURCE4 > %{buildroot}%{_libdir}/%{mpidir}/etc/mpivars.csh
chmod +x %{buildroot}%{_libdir}/%{mpidir}/etc/mpivars.*

# we need to force the compile mode via the wrapper-data.txt files
# (except on ia64 where the -m64 flag is not allowed by gcc)
%ifnarch ia64
for i in %{buildroot}%{_libdir}/%{mpidir}/share/openmpi/*wrapper-data.txt
do
  sed -e 's#compiler_flags=#compiler_flags='%{?modeflag}' #' < $i > $i.out
  mv $i.out $i
done
%endif

%clean
rm -rf %{buildroot}

%post
# Any and all upgrades/installs should remove all old dangling alternatives
# links except for the one case of openmpi-1.2.3 still installed because
# its %preun will fail if it can't remove the alternatives link itself.
# So, remove the link, and if 1.2.3 is installed, put it back.
/usr/sbin/alternatives --remove mpi-run %{_bindir}/orterun >/dev/null 2>&1
rpm -q openmpi-1.2.3.%{_arch} >/dev/null 2>&1
if [ "$?" -eq 0 ]; then
    # 1.2.3 is installed, recreate the link
    /usr/sbin/alternatives --install %{_bindir}/mpirun mpi-run \
	%{_bindir}/orterun 11 >/dev/null 2>&1
fi
# We can't fail
/bin/true


%post libs
# Any and all upgrades/installs should remove all old dangling alternatives
# links except for the one case of openmpi-1.2.3 still installed because
# its %preun will fail if it can't remove the alternatives link itself.
# So, remove the links, and if 1.2.3 is installed, put it back.
/usr/sbin/alternatives --remove mpilibs%{mode} \
    %{_libdir}/%{name}/%{name}.ld.conf >/dev/null 2>&1
/usr/sbin/alternatives --remove mpilibs%{mode} \
    %{_libdir}/%{name}/1.2.3-gcc/%{name}.ld.conf >/dev/null 2>&1
rpm -q openmpi-libs-1.2.3.%{_arch} > /dev/null 2>&1
if [ $? -eq 0 ]; then
    # 1.2.3 is installed, recreate the link
    /usr/sbin/alternatives --install \
	/etc/ld.so.conf.d/mpi%{mode}.conf mpilibs%{mode} \
	%{_libdir}/%{name}/1.2.3-gcc/%{name}.ld.conf 11 >/dev/null 2>&1
fi
# when upgrading 1.2.5, it expects a file in tmp to tell it what
# version we are...also it will fail to remove its mpi-selector
# registration, so we need to do that here
rpm -q openmpi-libs-1.2.5.%{_arch} > /dev/null 2>&1
if [ $? -eq 0 ]; then
    FILE=`mktemp /tmp/openmpi-upgrade-version.XXXXXX`
    echo "VERSION=%{version}" > $FILE
fi
# under certain specific circumstances, 1.2.5 will fail to remove its mpi
# registration.  so if we are an upgrade, we need to remove it, and even
# if it isn't installed we might need to remove it, so just remove it
# unilaterally
mpi-selector --unregister %{name}-1.2.5-gcc-%{_arch} --yes >/dev/null 2>&1
# It is safe to reregister the same setup at every upgrade/reinstall/etc.
mpi-selector --register %{name}-%{version}-%{opt_cc}-%{_arch} \
	--source-dir %{_libdir}/%{mpidir}/etc --yes
# We can't fail
/bin/true

%preun libs
# Count how many copies of this particular package are currently installed,
# knowing that on upgrade the new package being installed is already in this
# list by the time we are run, and that we are in the list too as we aren't
# removed until after our scripts complete.  If our count comes back as just
# 1, then we are the last copy of this particular version and we need to
# remove our registration.  This is true regardless of wheter this is an
# upgrade or an erase.  Note: we have to keep our arch in the query too or
# else multilib installations throw this count off.
COUNT=`rpm -q openmpi-%{version}.%{_arch} | wc -l`
if [ $COUNT -eq 1 ]; then
    mpi-selector --unregister %{name}-%{version}-%{opt_cc}-%{_arch} \
	--yes >/dev/null 2>&1
fi
# We can't fail
/bin/true

%post devel
# Any and all upgrades/installs should remove all old dangling alternatives
# links except for the one case of openmpi-1.2.3 still installed because
# its %preun will fail if it can't remove the alternatives link itself.
# So, remove the links, and if 1.2.3 is installed, put it back.
/usr/sbin/alternatives --remove mpicc \
    	%{_bindir}/opal_wrapper-%{mode} >/dev/null 2>&1
/usr/sbin/alternatives --remove mpicc \
    	%{_bindir}/opal_wrapper-1.2.3-gcc-%{mode} >/dev/null 2>&1
rpm -q openmpi-libs-1.2.3.%{_arch} > /dev/null 2>&1
if [ $? -eq 0 ]; then
    # 1.2.3 is installed, recreate the link
    /usr/sbin/alternatives --install %{_bindir}/mpicc mpicc \
	%{_bindir}/opal_wrapper-1.2.3-gcc-%{mode} 11 >/dev/null 2>&1
fi
# We can't fail
/bin/true

%files
%defattr(-,root,root,-)
%config(noreplace) %{_libdir}/%{mpidir}/etc/openmpi-*
%dir %{_libdir}/%{mpidir}/bin
%{_libdir}/%{mpidir}/bin/orte*
%{_libdir}/%{mpidir}/bin/ompi_info
%{_libdir}/%{mpidir}/bin/mpirun
%{_libdir}/%{mpidir}/bin/mpiexec
%{_libdir}/%{mpidir}/man/man1/orterun*
%{_libdir}/%{mpidir}/man/man1/mpirun*
%{_libdir}/%{mpidir}/man/man1/mpiexec*

%files libs
%defattr(-,root,root,-)
%dir %{_libdir}/%{mpidir}
%dir %{_libdir}/%{mpidir}/lib
%{_libdir}/%{mpidir}/etc/mpivars.*
%{_libdir}/%{mpidir}/lib/*.so.*
%{_libdir}/%{mpidir}/lib/%{name}/*.so
%dir %{_libdir}/%{mpidir}/share/openmpi
%{_libdir}/%{mpidir}/share/openmpi/*
%exclude %{_libdir}/%{mpidir}/share/openmpi/*-wrapper-data.txt

%files devel
%defattr(-,root,root,-)
%{_libdir}/%{mpidir}/bin/opal_wrapper
%{_libdir}/%{mpidir}/bin/mpic*
%{_libdir}/%{mpidir}/bin/mpif*
%{_libdir}/%{mpidir}/bin/mpiCC
%{_libdir}/%{mpidir}/man/man1/mpic*
%{_libdir}/%{mpidir}/man/man1/mpif*
%{_libdir}/%{mpidir}/man/man1/mpiC*
%dir %{_libdir}/%{mpidir}/include
%{_libdir}/%{mpidir}/include/*
%dir %{_libdir}/%{mpidir}/man/man3
%{_libdir}/%{mpidir}/man/man3/*
%{_libdir}/%{mpidir}/share/openmpi/*-wrapper-data.txt
%{_libdir}/%{mpidir}/lib/*.so
%{_libdir}/pkgconfig/%{name}-%{version}-%{opt_cc}.pc
%{_libdir}/%{mpidir}/lib/*.mod
%{_libdir}/%{mpidir}/lib/*.module


%changelog
* Fri Oct 03 2008 Doug Ledford <dledford@redhat.com> - 1.2.7-6
- Minor fix ups to the Requires(preun) and Requires(post)
- Add a Provides mpi to base package
- Resolves: bz465447

* Wed Sep 24 2008 Doug Ledford <dledford@redhat.com> - 1.2.7-5
- Minor fix for 1.2.5 mpi registration issues

* Tue Sep 23 2008 Doug Ledford <dledford@redhat.com> - 1.2.7-4
- Fix a few issues found during the package review for the openmpi11 package
- Resolves: bz465367

* Fri Sep 19 2008 Doug Ledford <dledford@redhat.com> - 1.2.7-3
- Touchup scripts for additional fixes related to upgrade paths

* Thu Sep 18 2008 Doug Ledford <dledford@redhat.com> - 1.2.7-2
- Missing a closing fi in a script, fixed

* Wed Sep 17 2008 Doug Ledford <dledford@redhat.com> - 1.2.7-1
- Update to latest upstream version
- Make the mandir be {mpidir}/man so that we don't need to futz with the
  manpath at all, man will automatically find our man pages for us
- buildrequire compat-dapl-devel so we find dapl and build a dapl transport
- Seriously rework the scriptlets to avoid the issue that was release noted
  with openmpi-1.2.5
- Resolves: bz447436, bz448388, bz455885, bz451469

* Wed Apr 09 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-5
- Fix the openib transport to know about both part ids on newer ehca adapters
- Resolves: bz441397

* Tue Apr 01 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-4
- Update to use mpi-selector for mpi package runtime priority selection
- Use mpi-selector change to fix bz433841 for real this time
- Related: bz428199

* Thu Feb 21 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-3
- Openmpi had the same bug as lam in regards to alternatives link management.
  Fix the bug by protecting the uninstall scriptlets from running unless we
  are doing an actual erase of the package.

* Tue Feb 05 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-2
- Don't require mpi-selector yet

* Sun Jan 27 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-1
- Update to latest release and use mpi-selector functionality
- Resolves: bz428199

* Mon Jul 16 2007 Doug Ledford <dledford@redhat.com> - 1.2.3-4
- Fix directory permissions on the base openmpi directory

* Thu Jul 12 2007 Florian La Roche <laroche@redhat.com> - 1.2.3-3
- requires alternatives for various sub-rpms
- Resolves: bz248029

* Mon Jul 02 2007 Doug Ledford <dledford@redhat.com> - 1.2.3-2
- Fix dangling symlink issue caused by a bad macro usage

* Wed Jun 27 2007 Doug Ledford <dledford@redhat.com> - 1.2.3-1
- Update to latest upstream version
- Compile against updated openib libraries
- Add udapl as a transport mechanism
- Fix file ownership on -libs package
- Take a swing at solving the multi-install compatibility issues
- Resolves: bz229298, bz245818

* Mon Feb 19 2007 Doug Ledford <dledford@redhat.com> - 1.1.1-7
- Bump version to be at least as high as the RHEL4U5 openmpi
- Integrate fixes made in RHEL4 openmpi into RHEL5 (fix a multilib conflict
  for the openmpi.module file by moving from _datadir to _libdir, make sure
  all sed replacements have the g flag so they replace all instances of
  the marker per line, not just the first, and add a %%defattr tag to the
  files section of the -libs package to avoid install errors about
  brewbuilder not being a user or group)
- Resolves: bz229298

* Wed Jan 17 2007 Doug Ledford <dledford@redhat.com> - 1.1.1-5
- Remove the FORTIFY_SOURCE and stack protect options
- Related: bz213075

* Fri Oct 20 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-4
- Bump and build against the final openib-1.1 package

* Wed Oct 18 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-3
- Fix an snprintf length bug in opal/util/cmd_line.c
- RESOLVES: rhbz#210714

* Wed Oct 18 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-2
- Bump and build against openib-1.1-0.pre1.1 instead of 1.0

* Tue Oct 17 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-1
- Update to upstream 1.1.1 version

* Fri Oct 13 2006 Doug Ledford <dledford@redhat.com> - 1.1-7
- ia64 can't take -m64 on the gcc command line, so don't set it there

* Wed Oct 11 2006 Doug Ledford <dledford@redhat.com> - 1.1-6
- Bump rev to match fc6 rev
- Fixup some issue with alternatives support
- Split the 32bit and 64bit libs ld.so.conf.d files into two files so
  multilib or single lib installs both work properly
- Put libs into their own package
- Add symlinks to /usr/share/openmpi/bin%{mode} so that opal_wrapper-%%{mode}
  can be called even if it isn't the currently selected default method in
  the alternatives setup (opal_wrapper needs to be called by mpicc, mpic++,
  etc. in order to determine compile mode from argv[0]).

* Sun Aug 27 2006 Doug Ledford <dledford@redhat.com> - 1.1-4
- Make sure the post/preun scripts only add/remove alternatives on initial
  install and final removal, otherwise don't touch.

* Fri Aug 25 2006 Doug Ledford <dledford@redhat.com> - 1.1-3
- Don't ghost the mpi.conf file as that means it will get removed when
  you remove 1 out of a number of alternatives based packages
- Put the .mod file in -devel

* Mon Aug  7 2006 Doug Ledford <dledford@redhat.com> - 1.1-2
- Various lint cleanups
- Switch to using the standard alternatives mechanism instead of a home
  grown one

* Wed Aug  2 2006 Doug Ledford <dledford@redhat.com> - 1.1-1
- Upgrade to 1.1
- Build with Infiniband support via openib

* Mon Jun 12 2006 Jason Vas Dias <jvdias@redhat.com> - 1.0.2-1
- Upgrade to 1.0.2

* Wed Feb 15 2006 Jason Vas Dias <jvdias@redhat.com> - 1.0.1-1
- Import into Fedora Core
- Resolve LAM clashes 

* Wed Jan 25 2006 Orion Poplawski <orion@cora.nwra.com> - 1.0.1-2
- Use configure options to install includes and libraries
- Add ld.so.conf.d file to find libraries
- Add -fPIC for x86_64

* Tue Jan 24 2006 Orion Poplawski <orion@cora.nwra.com> - 1.0.1-1
- 1.0.1
- Use alternatives

* Sat Nov 19 2005 Ed Hill <ed@eh3.com> - 1.0-2
- fix lam conflicts

* Fri Nov 18 2005 Ed Hill <ed@eh3.com> - 1.0-1
- initial specfile created

